package jesse.keeblarcraft.EventMgr;

import jesse.keeblarcraft.AttributeMgr.AttributeMgr;
import jesse.keeblarcraft.AttributeMgr.AttributeTree;
import jesse.keeblarcraft.ConfigMgr.GeneralConfig;
import jesse.keeblarcraft.Utils.DirectionalVec;
import net.fabricmc.fabric.api.networking.v1.PacketSender;
import net.minecraft.server.MinecraftServer;
import net.minecraft.server.network.ServerPlayNetworkHandler;
import net.minecraft.server.network.ServerPlayerEntity;
import net.minecraft.server.world.ServerWorld;
import net.minecraft.text.Text;

public class PlayerJoinListener {
    private static PlayerJoinListener static_inst;

    // Get the static instance for this class
    public static PlayerJoinListener GetInstance() {
        if (static_inst == null) {
            static_inst = new PlayerJoinListener();
        }
        return static_inst;
    }

    /////////////////////////////////////////////////////////////////////////////
    /// @fn         HandleServerJoinEvent
    ///
    /// @param[in]  handler is the network handler
    ///
    /// @param[in]  sender is the packet sender
    ///
    /// @param[in]  server is the MinecraftServer instance
    ///
    /// @brief      Callback function for player join event in minecraft server
    /////////////////////////////////////////////////////////////////////////////
    public void HandleServerJoinEvent(ServerPlayNetworkHandler handler, PacketSender sender, MinecraftServer server) {
        var player = handler.player;

        // Handle skill tree map instance
        if (AttributeMgr.activeTrees.containsKey(player.getUuidAsString()) == false) {
            AttributeMgr.activeTrees.put(player.getUuidAsString(), new AttributeTree(player.getUuidAsString()));
        }

        // Handle first time joining events (world spawn teleport, MOTD, etc)
        System.out.println("Running first time login stuff");
        IsFirstTimeLogin(player, server);
    }

    /////////////////////////////////////////////////////////////////////////////
    /// @fn         IsFirstTimeLogin
    ///
    /// @param[in]  player is the player who is joining
    ///
    /// @param[in]  server is the MinecraftServer instance
    ///
    /// @brief      Check to see if player is first time log-in. If so, we teleport
    ///             them to spawn. This function may be expanded in future
    /////////////////////////////////////////////////////////////////////////////
    private void IsFirstTimeLogin(ServerPlayerEntity player, MinecraftServer server) {
        // Send the MOTD + Spawn in global spawn
        player.sendMessage(Text.of(GeneralConfig.GetInstance().GetMOTD()));
        if (GeneralConfig.GetInstance().IsNewPlayer(player.getUuidAsString())) {
            DirectionalVec coords = GeneralConfig.GetInstance().GetSpawnCoords();
            if (coords != null) {
                for (ServerWorld world : player.getServer().getWorlds()) {
                    if (world.getRegistryKey().toString().equals(coords.world.toString())) {
                        try {
                            player.teleport(world, coords.x, coords.y, coords.z, coords.yaw, coords.pitch);
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    }
                }
            }
        }
    }
}
